/*
 * Project name:
     Receiver (ZIGBEE)
 * Copyright:
     (c) mikroElektronika, 2011.
 * Revision History:
     20110202:
       - Initial release (SZ);
 * Description:
     Many embedded systems need some amount of volatile storage for temporary
     data. Serial SRAMs are popular choice for volatile storage because of their
     small footprint, low I/O pin requirement, low-power consumption and low cost.
     This example writes and reads data from 23K640 Serial RAM device in byte
     read/write operation mode using standard SPI communication. For more
     information about available operating modes consult 23K640 datasheet:
     http://ww1.microchip.com/downloads/en/DeviceDoc/22126C.pdf
 * Test configuration:
     MCU:             PIC24FJ16KA102
                      http://ww1.microchip.com/downloads/en/DeviceDoc/PIC24F16KA102_Family_datasheet_39927b.pdf
     Dev.Board:       Easy24-33 v6
                      http://www.mikroe.com/eng/products/view/494/easy24-33-v6-development-system/
     Oscillator:      HS, 8.00000 MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/eng/products/view/231/mikroc-pro-for-dspic30-33-and-pic24/
 * NOTES:
     - Turn ON: SW4.3, SW4.4 SW4.5, SW4.6, SW3.3, SW3.5 and SW3.7.
     - Turn on the LCD backlight (SW3.8).
*/

#include "registers.h"

// LCD module connections
sbit LCD_RS at LATB4_bit;
sbit LCD_EN at LATB5_bit;
sbit LCD_D4 at LATB6_bit;
sbit LCD_D5 at LATA0_bit;
sbit LCD_D6 at LATA1_bit;
sbit LCD_D7 at LATA4_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB6_bit;
sbit LCD_D5_Direction at TRISA0_bit;
sbit LCD_D6_Direction at TRISA1_bit;
sbit LCD_D7_Direction at TRISA4_bit;
// End LCD module connections

// EasyBee3 connections
sbit CS at RB15_bit;                 //CS pin
sbit RST at RB0_bit;                 //RST pin
sbit INT at RB7_bit;                 //INT pin
sbit WAKE_ at RB1_bit;               //WAKE pin

sbit CS_Direction at TRISB15_bit;    //CS pin direction
sbit RST_Direction at TRISB0_bit;    //RST pin direction
sbit INT_Direction at TRISB7_bit;    //INT pin direction
sbit WAKE_Direction at TRISB1_bit;   //WAKE pin direction
// End EasyBee3 connections



char txt[4];
const unsigned short int DATA_LENGHT = 3;
const unsigned short int HEADER_LENGHT = 11;

int address_RX_FIFO = 0x300;
int address_TX_normal_FIFO = 0x000;
short int data_RX_FIFO[1 + HEADER_LENGHT + DATA_LENGHT + 2 + 1 + 1], lost_data = 0;

short int ADDRESS_short_1[] = {1,1};               // Source address
short int ADDRESS_long_1[]  = {1,1,1,1,1,1,1,1};
short int ADDRESS_short_2[] = {2,2};               // Destination address
short int ADDRESS_long_2[]  = {2,2,2,2,2,2,2,2};
short int PAN_ID_1[] = {3,3};                      // Source PAN ID
short int PAN_ID_2[] = {3,3};                      // Destination PAN ID

short int DATA_RX[DATA_LENGHT], DATA_TX[DATA_LENGHT], data_TX_normal_FIFO[DATA_LENGHT + HEADER_LENGHT + 2];
short int LQI = 0, RSSI2 = 0, SEQ_NUMBER = 0x23;

/*
* Functions for reading and writing registers in short address memory space
*/

// write data in short address register
void write_ZIGBEE_short(short int address, short int data_r){
 CS = 0;

 address = ((address << 1) & 0b01111111) | 0x01; // calculating addressing mode
 SPI1_Write(address);                            // addressing register
 SPI1_Write(data_r);                             // write data in register

 CS = 1;
}

// read data from short address register
short int read_ZIGBEE_short(short int address)
{
  short int data_r = 0, dummy_data_r = 0;

  CS = 0;

 address = (address << 1) & 0b01111110;          // calculating addressing mode
 SPI1_Write(address);                            // addressing register
 data_r = SPI1_Read(dummy_data_r);               // read data from register

  CS = 1;
  return data_r;
}

/*
* Functions for reading and writing registers in long address memory space
*/

// Write data in long address register
void write_ZIGBEE_long(int address, short int data_r){
  short int address_high = 0, address_low = 0;

  CS = 0;

  address_high = (((short int)(address >> 3)) & 0b01111111) | 0x80;  // calculating addressing mode
  address_low  = (((short int)(address << 5)) & 0b11100000) | 0x10;  // calculating addressing mode
  SPI1_Write(address_high);                                          // addressing register
  SPI1_Write(address_low);                                           // addressing register
  SPI1_Write(data_r);                                                // write data in registerr

  CS = 1;
}

// Read data from long address register
short int read_ZIGBEE_long(int address){
  short int data_r = 0, dummy_data_r = 0;
  short int address_high = 0, address_low = 0;

  CS = 0;

  address_high = ((short int)(address >> 3) & 0b01111111) | 0x80;  // calculating addressing mode
  address_low  = ((short int)(address << 5) & 0b11100000);         // calculating addressing mode
  SPI1_Write(address_high);                                        // addressing register
  SPI1_Write(address_low);                                         // addressing register
  data_r = SPI1_Read(dummy_data_r);                                // read data from register

  CS = 1;
  return data_r;
}

/*
* Reset functions
*/

// Reset from pin
void pin_reset(void)
{
  RST = 0;  // activate reset
  Delay_ms(5);
  RST = 1;  // deactivate reset
  Delay_ms(5);
}

void PWR_reset(void){
  write_ZIGBEE_short(SOFTRST, 0x04);     //0x04  mask for RSTPWR bit
}

void BB_reset(void){
  write_ZIGBEE_short(SOFTRST, 0x02);     //0x02 mask for RSTBB bit
}

void MAC_reset(void){
  write_ZIGBEE_short(SOFTRST, 0x01);     //0x01 mask for RSTMAC bit
}

void software_reset(void){               //PWR_reset,BB_reset and MAC_reset at once
  write_ZIGBEE_short(SOFTRST, 0x07);
}

void RF_reset(void){
  short int temp = 0;
  temp = read_ZIGBEE_short(RFCTL);
  temp = temp | 0x04;                    //mask for RFRST bit
  write_ZIGBEE_short(RFCTL, temp);
  temp = temp & (!0x04);                 //mask for RFRST bit
  write_ZIGBEE_short(RFCTL, temp);
  Delay_ms(1);
}

/*
*  Interrupt
*/
void enable_interrupt(void)
{
 write_ZIGBEE_short(INTCON_M, 0x00);     //0x00  all interrupts are enable
}

/*
*  Set channel
*/
void set_channel(short int channel_number){               // 11-26 possible channels
  if((channel_number > 26) || (channel_number < 11)) channel_number = 11;
  switch(channel_number){
    case 11:
    write_ZIGBEE_long(RFCON0, 0x02);  //0x02 for 11. channel
    break;
    case 12:
    write_ZIGBEE_long(RFCON0, 0x12);  //0x12 for 12. channel
    break;
    case 13:
    write_ZIGBEE_long(RFCON0, 0x22);  //0x22 for 13. channel
    break;
    case 14:
    write_ZIGBEE_long(RFCON0, 0x32);  //0x32 for 14. channel
    break;
    case 15:
    write_ZIGBEE_long(RFCON0, 0x42);  //0x42 for 15. channel
    break;
    case 16:
    write_ZIGBEE_long(RFCON0, 0x52);  //0x52 for 16. channel
    break;
    case 17:
    write_ZIGBEE_long(RFCON0, 0x62);  //0x62 for 17. channel
    break;
    case 18:
    write_ZIGBEE_long(RFCON0, 0x72);  //0x72 for 18. channel
    break;
    case 19:
    write_ZIGBEE_long(RFCON0, 0x82);  //0x82 for 19. channel
    break;
    case 20:
    write_ZIGBEE_long(RFCON0, 0x92);  //0x92 for 20. channel
    break;
    case 21:
    write_ZIGBEE_long(RFCON0, 0xA2);  //0xA2 for 21. channel
    break;
    case 22:
    write_ZIGBEE_long(RFCON0, 0xB2);  //0xB2 for 22. channel
    break;
    case 23:
    write_ZIGBEE_long(RFCON0, 0xC2);  //0xC2 for 23. channel
    break;
    case 24:
    write_ZIGBEE_long(RFCON0, 0xD2);  //0xD2 for 24. channel
    break;
    case 25:
    write_ZIGBEE_long(RFCON0, 0xE2);  //0xE2 for 25. channel
    break;
    case 26:
    write_ZIGBEE_long(RFCON0, 0xF2);  //0xF2 for 26. channel
    break;
  }
  RF_reset();
}

/*
*  Set CCA mode
*/
void set_CCA_mode(short int CCA_mode){
  short int temp = 0;
  switch(CCA_mode){
    case 1:                                  //ENERGY ABOVE THRESHOLD
    {
      temp = read_ZIGBEE_short(BBREG2);
      temp = temp | 0x80;                    //0x80 mask
      temp = temp & 0xDF;                    //0xDF mask
      write_ZIGBEE_short(BBREG2, temp);

      write_ZIGBEE_short(CCAEDTH, 0x60);    //Set CCA ED threshold to -69 dBm
    }
    break;

    case 2:                                 //CARRIER SENSE ONLY
    {
      temp = read_ZIGBEE_short(BBREG2);
      temp = temp | 0x40;                   // 0x40 mask
      temp = temp & 0x7F;                   // 0x7F mask
      write_ZIGBEE_short(BBREG2, temp);

      temp = read_ZIGBEE_short(BBREG2);     // carrier sense threshold
      temp = temp | 0x38;
      temp = temp & 0xFB;
      write_ZIGBEE_short(BBREG2, temp);
    }
    break;

    case 3:                                 //CARRIER SENSE AND ENERGY ABOVE THRESHOLD
    {
      temp = read_ZIGBEE_short(BBREG2);
      temp = temp | 0xC0;                   //0xC0 mask
      write_ZIGBEE_short(BBREG2, temp);

      temp = read_ZIGBEE_short(BBREG2);     // carrier sense threshold
      temp = temp | 0x38;                   // 0x38 mask
      temp = temp & 0xFB;                   // 0xFB mask
      write_ZIGBEE_short(BBREG2, temp);

      write_ZIGBEE_short(CCAEDTH, 0x60);    //Set CCA ED threshold to -69 dBm
    }
    break;

  }

}

/*
* Set RSSI mode
*/
void set_RSSI_mode(short int RSSI_mode){// 1 for RSSI1, 2 for RSSI2 mode
  short int temp = 0;

  switch(RSSI_mode){
    case 1:
    {
      temp = read_ZIGBEE_short(BBREG6);
      temp = temp | 0x80;               //0x80 mask for RSSI1 mode
      write_ZIGBEE_short(BBREG6, temp);
    }
    break;

    case 2:
    write_ZIGBEE_short(BBREG6, 0x40);   //0x40 data for RSSI2 mode
    break;
  }
}

/*
* Set type of device
*/
void nonbeacon_PAN_coordinator_device(void){
  short int temp = 0;

  temp = read_ZIGBEE_short(RXMCR);
  temp = temp | 0x08;                 // 0x08 mask for PAN coordinator
  write_ZIGBEE_short(RXMCR, temp);

  temp = read_ZIGBEE_short(TXMCR);
  temp = temp & 0xDF;                 // 0xDF mask for CSMA-CA mode
  write_ZIGBEE_short(TXMCR, temp);

  write_ZIGBEE_short(ORDER, 0xFF);    // BO, SO are 15
}

void nonbeacon_coordinator_device(void){
  short int temp = 0;

  temp = read_ZIGBEE_short(RXMCR);
  temp = temp | 0x04;                 // 0x04 mask for coordinator
  write_ZIGBEE_short(RXMCR, temp);

  temp = read_ZIGBEE_short(TXMCR);
  temp = temp & 0xDF;                 // 0xDF mask for CSMA-CA mode
  write_ZIGBEE_short(TXMCR, temp);

  write_ZIGBEE_short(ORDER, 0xFF);    // BO, SO  are 15
}

void nonbeacon_device(void){
  short int temp = 0;

  temp = read_ZIGBEE_short(RXMCR);
  temp = temp & 0xF3;             // 0xF3 mask for PAN coordinator and coordinator
  write_ZIGBEE_short(RXMCR, temp);

  temp = read_ZIGBEE_short(TXMCR);
  temp = temp & 0xDF;             // 0xDF mask for CSMA-CA mode
  write_ZIGBEE_short(TXMCR, temp);
}

/*
* ACK request
*/
void set_ACK(void){
  short int temp = 0;

  temp = read_ZIGBEE_short(TXNCON);
  temp = temp | 0x04;                    //0x04 mask for set ACK
  write_ZIGBEE_short(TXNCON, temp);
}

void set_not_ACK(void){
  short int temp = 0;

  temp = read_ZIGBEE_short(TXNCON);
  temp = temp & (!0x04);                 //0x04 mask for set not ACK
  write_ZIGBEE_short(TXNCON, temp);
}

/*
*  Encrypt
*/
void set_encrypt(void){
  short int temp = 0;

  temp = read_ZIGBEE_short(TXNCON);
  temp = temp | 0x02;                   // mask for set encrypt
  write_ZIGBEE_short(TXNCON, temp);
}

void set_not_encrypt(void){
  short int temp = 0;

  temp = read_ZIGBEE_short(TXNCON);
  temp = temp & (!0x02);                // mask for set not encrypt
  write_ZIGBEE_short(TXNCON, temp);
}

/*
* Transmit packet
*/
void start_transmit(void){
  short int temp = 0;

  temp = read_ZIGBEE_short(TXNCON);
  temp = temp | 0x01;                // mask for start transmit
  write_ZIGBEE_short(TXNCON, temp);
}

/*
* Interframe spacing
*/
void set_IFS_recomended(void){
  short int temp = 0;

  write_ZIGBEE_short(RXMCR, 0x93);       // Min SIFS Period

  temp = read_ZIGBEE_short(TXPEND);
  temp = temp | 0x7C;                    // MinLIFSPeriod
  write_ZIGBEE_short(TXPEND, temp);

  temp = read_ZIGBEE_short(TXSTBL);
  temp = temp | 0x90;                    // MinLIFSPeriod
  write_ZIGBEE_short(TXSTBL, temp);

  temp = read_ZIGBEE_short(TXTIME);
  temp = temp | 0x31;                    // TurnaroundTime
  write_ZIGBEE_short(TXTIME, temp);
}

void set_IFS_default(void){
  short int temp = 0;

  write_ZIGBEE_short(RXMCR, 0x75);       // Min SIFS Period

  temp = read_ZIGBEE_short(TXPEND);
  temp = temp | 0x84;                    // Min LIFS Period
  write_ZIGBEE_short(TXPEND, temp);

  temp = read_ZIGBEE_short(TXSTBL);
  temp = temp | 0x50;                    // Min LIFS Period
  write_ZIGBEE_short(TXSTBL, temp);

  temp = read_ZIGBEE_short(TXTIME);
  temp = temp | 0x41;                    // Turnaround Time
  write_ZIGBEE_short(TXTIME, temp);
}

/*
* Reception mode
*/
void set_reception_mode(short int r_mode){    // 1 normal, 2 error, 3 promiscuous mode
  short int temp = 0;

  switch(r_mode)
  {
   case 1:
   {
    temp = read_ZIGBEE_short(RXMCR);      // normal mode
    temp = temp & (!0x03);                // mask for normal mode
    write_ZIGBEE_short(RXMCR, temp);
   }
   break;

   case 2:
   {
    temp = read_ZIGBEE_short(RXMCR);      // error mode
    temp = temp & (!0x01);                // mask for error mode
    temp = temp | 0x02;                   // mask for error mode
    write_ZIGBEE_short(RXMCR, temp);
   }
   break;

   case 3:
   {
    temp = read_ZIGBEE_short(RXMCR);      // promiscuous mode
    temp = temp & (!0x02);                // mask for promiscuous mode
    temp = temp | 0x01;                   // mask for promiscuous mode
    write_ZIGBEE_short(RXMCR, temp);
   }
   break;
  }
}

/*
*  Frame format filter
*/
void set_frame_format_filter(short int fff_mode){   // 1 all frames, 2 command only, 3 data only, 4 beacon only
  short int temp = 0;

  switch(fff_mode)
  {
   case 1:
   {
    temp = read_ZIGBEE_short(RXFLUSH);      // all frames
    temp = temp & (!0x0E);                  // mask for all frames
    write_ZIGBEE_short(RXFLUSH, temp);
   }
   break;

   case 2:
   {
    temp = read_ZIGBEE_short(RXFLUSH);      // command only
    temp = temp & (!0x06);                  // mask for command only
    temp = temp | 0x08;                     // mask for command only
    write_ZIGBEE_short(RXFLUSH, temp);
   }
   break;

   case 3:
   {
    temp = read_ZIGBEE_short(RXFLUSH);      // data only
    temp = temp & (!0x0A);                  // mask for data only
    temp = temp | 0x04;                     // mask for data only
    write_ZIGBEE_short(RXFLUSH, temp);
   }
   break;

   case 4:
   {
    temp = read_ZIGBEE_short(RXFLUSH);      // beacon only
    temp = temp & (!0x0C);                  // mask for beacon only
    temp = temp | 0x02;                     // mask for beacon only
    write_ZIGBEE_short(RXFLUSH, temp);
   }
   break;
  }
}

/*
*  Flush RX FIFO pointer
*/
void   flush_RX_FIFO_pointer(void){
  short int temp;

  temp = read_ZIGBEE_short(RXFLUSH);
  temp = temp | 0x01;                    // mask for flush RX FIFO
  write_ZIGBEE_short(RXFLUSH, temp);
}

/*
* FIFO
*/
void read_RX_FIFO(void){
  unsigned short int temp = 0;
  int i = 0;

  temp = read_ZIGBEE_short(BBREG1);      // disable receiving packets off air.
  temp = temp | 0x04;                    // mask for disable receiving packets
  write_ZIGBEE_short(BBREG1, temp);

  for(i=0; i<128; i++)
  {
   if(i <  (1 + DATA_LENGHT + HEADER_LENGHT + 2 + 1 + 1))
     data_RX_FIFO[i] = read_ZIGBEE_long(address_RX_FIFO + i);  // reading valid data from RX FIFO
   if(i >= (1 + DATA_LENGHT + HEADER_LENGHT + 2 + 1 + 1))
     lost_data = read_ZIGBEE_long(address_RX_FIFO + i);        // reading invalid data from RX FIFO
  }

  DATA_RX[0] = data_RX_FIFO[HEADER_LENGHT + 1];               // coping valid data
  DATA_RX[1] = data_RX_FIFO[HEADER_LENGHT + 2];               // coping valid data
  DATA_RX[2] = data_RX_FIFO[HEADER_LENGHT + 3];               // coping valid data
  LQI   = data_RX_FIFO[1 + HEADER_LENGHT + DATA_LENGHT + 2];  // coping valid data
  RSSI2 = data_RX_FIFO[1 + HEADER_LENGHT + DATA_LENGHT + 3];  // coping valid data

  temp = read_ZIGBEE_short(BBREG1);      // enable receiving packets off air.
  temp = temp & (!0x04);                 // mask for enable receiving
  write_ZIGBEE_short(BBREG1, temp);

}

void write_TX_normal_FIFO(void){
   int i = 0;

   data_TX_normal_FIFO[0]  = HEADER_LENGHT;
   data_TX_normal_FIFO[1]  = HEADER_LENGHT + DATA_LENGHT;
   data_TX_normal_FIFO[2]  = 0x01;                        // control frame
   data_TX_normal_FIFO[3]  = 0x88;
   //data_TX_normal_FIFO[4]  = SEQ_NUMBER;                // sequence number
   data_TX_normal_FIFO[5]  = PAN_ID_2[1];                 // destinatoin pan
   data_TX_normal_FIFO[6]  = PAN_ID_2[0];
   data_TX_normal_FIFO[7]  = ADDRESS_short_2[0];          // destination address
   data_TX_normal_FIFO[8]  = ADDRESS_short_2[1];
   data_TX_normal_FIFO[9]  = PAN_ID_1[0];                 // source pan
   data_TX_normal_FIFO[10] = PAN_ID_1[1];
   data_TX_normal_FIFO[11] = ADDRESS_short_1[0];          // source address
   data_TX_normal_FIFO[12] = ADDRESS_short_1[1];

   data_TX_normal_FIFO[13] = DATA_TX[0];                  // data
   data_TX_normal_FIFO[14] = DATA_TX[1];
   data_TX_normal_FIFO[15] = DATA_TX[2];

   for(i = 0; i < (HEADER_LENGHT + DATA_LENGHT + 2); i++)
   {
    write_ZIGBEE_long(address_TX_normal_FIFO + i, data_TX_normal_FIFO[i]); // write frame into normal FIFO
   }

   set_not_ACK();
   set_not_encrypt();
   start_transmit();

}

/*
* Address
*/
void set_short_address(short int * address){
  write_ZIGBEE_short(SADRL, address[0]);
  write_ZIGBEE_short(SADRH, address[1]);
}

void set_long_address(short int * address){
  short int i = 0;

  for(i = 0; i < 8; i++)
  {
   write_ZIGBEE_short(EADR0 + i, address[i]);   // 0x05 address of EADR0
  }
}

void set_PAN_ID(short int * address){
  write_ZIGBEE_short(PANIDL, address[0]);
  write_ZIGBEE_short(PANIDH, address[1]);
}

/*
* Wake
*/
void set_wake_from_pin(void){
  short int temp = 0;

  WAKE_ = 0;
  temp = read_ZIGBEE_short(RXFLUSH);
  temp = temp | 0x60;                     // mask
  write_ZIGBEE_short(RXFLUSH, temp);

  temp = read_ZIGBEE_short(WAKECON);
  temp = temp | 0x80;
  write_ZIGBEE_short(WAKECON, temp);
}

void pin_wake(void){
  WAKE_ = 1;
  Delay_ms(5);
}

/*
* PLL
*/
void enable_PLL(void){
   write_ZIGBEE_long(RFCON2, 0x80);       // mask for PLL enable
}

void disable_PLL(void){
   write_ZIGBEE_long(RFCON2, 0x00);       // mask for PLL disable
}

/*
* Tx power
*/
void set_TX_power(unsigned short int power){              // 0-31 possible variants
  if((power < 0) || (power > 31)) power = 31;
  power = 31 - power;                                     //0 max, 31 min -> 31 max, 0 min
  power = ((power & 0b00011111) << 3) & 0b11111000;       // calculating power
  write_ZIGBEE_long(RFCON3, power);
}

/*
* Init ZIGBEE module
*/
void init_ZIGBEE_basic(void){
   write_ZIGBEE_short(PACON2, 0x98);   // Initialize FIFOEN = 1 and TXONTS = 0x6
   write_ZIGBEE_short(TXSTBL, 0x95);   // Initialize RFSTBL = 0x9
   write_ZIGBEE_long(RFCON1, 0x01);    // Initialize VCOOPT = 0x01
   enable_PLL();                       // Enable PLL (PLLEN = 1)
   write_ZIGBEE_long(RFCON6, 0x90);    // Initialize TXFIL = 1 and 20MRECVR = 1
   write_ZIGBEE_long(RFCON7, 0x80);    // Initialize SLPCLKSEL = 0x2 (100 kHz Internal oscillator)
   write_ZIGBEE_long(RFCON8, 0x10);    // Initialize RFVCO = 1
   write_ZIGBEE_long(SLPCON1, 0x21);   // Initialize CLKOUTEN = 1 and SLPCLKDIV = 0x01
}

void init_ZIGBEE_nonbeacon(void){
 init_ZIGBEE_basic();
 set_CCA_mode(1);     // Set CCA mode to ED and set threshold
 set_RSSI_mode(2);    // RSSI2 mode
 enable_interrupt();  // Enables all interrupts
 set_channel(11);     // Channel 11
 RF_reset();
}

char debauns_INTn(void){
  char i = 0, j = 0, intn_d = 0;
  for(i = 0; i < 5; i++)
  {
     intn_d = INT;
     if (intn_d == 1) j++;
  }
  if(j > 2) return 1;
  else return 0;
}


/*
*  Main
*/

void main(){
  short int temp1 = 0;
  AD1PCFG = 0x1C3F;                       // Configure AN pins as digital I/O

  CS_Direction = 0;                       // Set direction to be output
  RST_Direction  = 0;                     // Set direction to be output
  INT_Direction  = 1;                     // Set direction to be input
  WAKE_Direction = 0;                     // Set direction to be output

  Delay_ms(5);

  SPI1_Init();                            // Initialize SPI module
  pin_reset();                            // Activate reset from pin
  software_reset();                       // Activate software reset
  RF_reset();                             // RF reset
  set_WAKE_from_pin();                    // Set wake from pin

  set_long_address(ADDRESS_long_2);       // Set long address
  set_short_address(ADDRESS_short_2);     // Set short address
  set_PAN_ID(PAN_ID_2);                   // Set PAN_ID

  init_ZIGBEE_nonbeacon();                // Initialize ZigBee module
  nonbeacon_PAN_coordinator_device();
  set_TX_power(31);                       // Set max TX power
  set_frame_format_filter(1);             // 1 all frames, 3 data frame only
  set_reception_mode(1);                  // 1 normal mode

  pin_wake();                             // Wake from pin

  Lcd_Init();                             // Initialize LCD
  Lcd_Cmd(_LCD_CLEAR);                    // Clear display
  Lcd_Cmd(_LCD_CURSOR_OFF);               // Cursor off
  Lcd_Out(1,1,"Received :");              // Write on LCD

  ByteToStr(0,&txt);

  while(1){                               // Infinite loop
    if( debauns_INTn() == 0 ){            // Polling line INT
      temp1 = read_ZIGBEE_short(INTSTAT); // Read and flush register INTSTAT
      read_RX_FIFO();                     // Read receive data
      ByteToStr(DATA_RX[2],&txt);         // Convert third byte to string
      Lcd_Chr(2,1,DATA_RX[0]);            // Display first received byte
      Lcd_Chr(2,2,DATA_RX[1]);            // Display second received byte
      Lcd_Out(2,5,txt);                   // Display third received byte
    }
  }
}